﻿/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Creates SQL objects used by Customer Service Alerts

set nocount on;

print 'Creating table dbo.tcsa_AlertDefinitions';
go

if object_id('tcsa_AlertDefinitions') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_AlertDefinitions') and indid < 2) > 0
	begin;
		exec sp_rename tcsa_AlertDefinitions, tcsa_AlertDefinitions_CSAInstallTemp;
	end;
	else
	begin;
		drop table dbo.tcsa_AlertDefinitions;
	end;
end;
go
if object_id('pkcsa_AlertDefinitions') is not NULL
begin;
	alter table dbo.tcsa_AlertDefinitions_CSAInstallTemp
		drop constraint pkcsa_AlertDefinitions;
end;
go
if object_id('tcsa_AlertDefinitions_CSAInstallTemp') is not NULL
begin;
	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertDefinitions_CSAInstallTemp' and name = 'RuleGroup')
	begin;
		alter table dbo.tcsa_AlertDefinitions_CSAInstallTemp
			add RuleGroup varchar(100) not NULL default '';
	end;

	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertDefinitions_CSAInstallTemp' and name = 'ALERT_ID')
	begin;
		alter table dbo.tcsa_AlertDefinitions_CSAInstallTemp
			add ALERT_ID varchar(20) not NULL default '';
	end;
end;
go

if object_id('tcsa_status_rules') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_status_rules') and indid < 2) > 0
	begin;
		exec sp_rename tcsa_status_rules, tcsa_status_rules_CSAInstallTemp;
	end;
	else
	begin;
		drop table dbo.tcsa_status_rules;
	end;
end;
go
if object_id('pkcsa_status_rules') is not NULL
begin;
	alter table dbo.tcsa_status_rules_CSAInstallTemp
		drop constraint pkcsa_status_rules;
end;
go
if object_id('tcsa_status_rules_CSAInstallTemp') is not NULL
begin;
	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_status_rules_CSAInstallTemp' and name = 'RuleGroup')
	begin;
		alter table dbo.tcsa_status_rules_CSAInstallTemp
			add RuleGroup varchar(100) not NULL default '';
	end;

	if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_status_rules_CSAInstallTemp' and name = 'ALERT_ID')
	begin;
		alter table dbo.tcsa_status_rules_CSAInstallTemp
			add ALERT_ID varchar(20) not NULL default '';
	end;
end;
go

create table dbo.tcsa_AlertDefinitions
(
	SEQN int identity
		constraint pkcsa_AlertDefinitions primary key clustered,
	CRITERIA varchar(3072) not NULL default '',
	ENABLED bit not NULL default 1,
	DESCRIPTION varchar(255) not NULL default '',
	ALERT_TYPE varchar(10) not NULL default 'SQL',
	ALERT_ID varchar(20) not NULL default '',
	RuleGroup varchar(100) not NULL default ''
);
go
grant select, insert, update, delete on dbo.tcsa_AlertDefinitions to iMIS;
go

if object_id('tcsa_status_rules_CSAInstallTemp') is not NULL
begin;
	declare @oldrows int,
		@newrows int;

	insert dbo.tcsa_AlertDefinitions (CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
		select CRITERIA, ENABLED, '', 'SQL', ALERT_ID, RuleGroup
		from dbo.tcsa_status_rules_CSAInstallTemp
		order by RuleGroup, SEQN;

	set @newrows = @@rowcount;

	select @oldrows = count(*)
		from dbo.tcsa_status_rules_CSAInstallTemp;

	if @oldrows = @newrows
	begin;
		drop table dbo.tcsa_status_rules_CSAInstallTemp;
	end;
	else
	begin;
		print '!!! Error copying data from tcsa_status_rules_CSAInstallTemp into new table';
	end;
end;
go
if object_id('tcsa_AlertDefinitions_CSAInstallTemp') is not NULL
begin;
	declare @oldrows int,
		@newrows int;

	insert dbo.tcsa_AlertDefinitions (CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
		select CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup
		from dbo.tcsa_AlertDefinitions_CSAInstallTemp
		order by RuleGroup, SEQN;

	set @newrows = @@rowcount;

	select @oldrows = count(*)
		from dbo.tcsa_AlertDefinitions_CSAInstallTemp;

	if @oldrows = @newrows
	begin;
		drop table dbo.tcsa_AlertDefinitions_CSAInstallTemp;
	end;
	else
	begin;
		print '!!! Error copying data from tcsa_AlertDefinitions_CSAInstallTemp into new table';
	end;
end;
go
exec sp_recompile 'tcsa_AlertDefinitions';
go

print 'Creating table tcsa_AlertOutput (used by Xtender)';
go
if object_id('tcsa_Status_Output') is not NULL
begin;
	-- Always ok to delete
	drop table dbo.tcsa_Status_Output;
end;
go
if object_id('tcsa_AlertOutput') is not NULL
begin;
	-- Always ok to delete
	drop table dbo.tcsa_AlertOutput;
end;
go
create table dbo.tcsa_AlertOutput
(
	SEQN int identity
		constraint pkcsa_AlertOutput primary key clustered,
	OWNER_SPID smallint not NULL default @@spid,
	IMIS_USER varchar(50) not NULL default user_name(),
	EVENT_TYPE varchar(15) not NULL default 'ID',
	KEY_VALUE varchar(31) not NULL default '',
	RANK smallint not NULL default 0,						-- main window
	ICON smallint not NULL default 4,						-- detail
	KEYWORD varchar(40) not NULL default '',				-- detail
	DESCRIPTION varchar(4000) not NULL default '',			-- detail
	TOOLTIP_TEXT varchar(1000) not NULL default '',			-- detail
	TITLE_TEXT varchar(255) not NULL default '',			-- detail
	CLIPBOARD_TEXT text NOT NULL default '',				-- detail
	CLIPBOARD_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TEXT varchar(1000) not NULL default '',			-- detail
	ACTION_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TYPE varchar(30) not NULL default 'Launch',		-- detail
	LOG_TEXT varchar(1000) not NULL default '',				-- main window
	PLAY_SOUND bit not NULL default 0,						-- main window
	AUTO_POPUP bit not NULL default 0,						-- detail
	AUTO_CLIPBOARD bit not NULL default 0,					-- detail
	AUTO_ACTION bit not NULL default 0						-- detail
);
go
create index icsa_AlertOutputUSERRANK
	on dbo.tcsa_AlertOutput(OWNER_SPID, RANK);
go
grant select, insert, update, delete on dbo.tcsa_AlertOutput to iMIS;
go

print 'Creating table tcsa_AlertUsageUrl (used by iPart)';
go
-- Enable later if needed
/*
if object_id('tcsa_AlertUsageUrl') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_AlertUsageUrl') and indid < 2) > 0
	begin;
		if object_id('dbo.FK_AlertUsage_Url') is not null
		begin
			alter table dbo.tcsa_AlertUsage
				drop constraint FK_AlertUsage_Url;
		end

		if object_id('dbo.pkcsa_AlertUsageUrl') is not null
		begin
			alter table dbo.tcsa_AlertUsageUrl
				drop constraint pkcsa_AlertUsageUrl;
		end

		exec sp_rename tcsa_AlertUsageUrl, tcsa_AlertUsageUrl_CSAInstallTemp;
	end;
	else
	begin;
		if object_id('dbo.FK_AlertUsage_URL') is not null
		begin
			alter table dbo.tcsa_AlertUsage
				drop constraint FK_AlertUsage_Url;
		end

		drop table dbo.tcsa_AlertUsageUrl;
	end;
end;
*/
go
if object_id('tcsa_AlertUsageUrl') is NULL
begin
	create table dbo.tcsa_AlertUsageUrl
	(
		AlertUsageUrlKey int identity
			constraint pkcsa_AlertUsageUrl primary key nonclustered,
		Url nvarchar(1000) not NULL,
		Digest as case
				when len(Url) <= 40 then left(Url, 40)
				else left(Url, 15) + right(Url, 25)
			end
	);
end
go
grant select, insert, update, delete on dbo.tcsa_AlertUsageUrl to iMIS;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_AlertUsageUrl')
		and name = 'icsa_AlertUsageUrlDigest')
begin
	create clustered index icsa_AlertUsageUrlDigest
		on dbo.tcsa_AlertUsageUrl(Digest);
end;
go

print 'Creating table tcsa_AlertUsage (used by iPart)';
go
if object_id('tcsa_AlertUsage') is not NULL
	and exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertUsage' and name = 'Url')
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_AlertUsage') and indid < 2) > 0
	begin;
		if object_id('dbo.pkcsa_AlertUsage') is not null
		begin
			alter table dbo.tcsa_AlertUsage
				drop constraint pkcsa_AlertUsage;
		end

		if object_id('dbo.FK_AlertUsage_Url') is not null
		begin
			alter table dbo.tcsa_AlertUsage
				drop constraint FK_AlertUsage_Url;
		end

		exec sp_rename tcsa_AlertUsage, tcsa_AlertUsage_CSAInstallTemp;
	end;
	else
	begin;
		drop table dbo.tcsa_AlertUsage;
	end;
end;
go
if object_id('tcsa_AlertUsage') is NULL
begin
	create table dbo.tcsa_AlertUsage
	(
		SEQN int identity
			constraint pkcsa_AlertUsage primary key clustered,
		EventStart datetime not NULL,
		EventEnd datetime not NULL default getdate(),
		RuleGroup varchar(100) not NULL,
		RuleCount int not NULL,
		RowsCount int not NULL,
		ErrorCount int not NULL,
		ActionCount int not NULL,
		AutoActionCount int not NULL,
		ID varchar(10) not NULL,
		UserId varchar(60) not NULL,
		AlertUsageUrlKey int not NULL
			constraint FK_AlertUsage_Url
			references tcsa_AlertUsageUrl(AlertUsageUrlKey)
	);
end
go
grant select, insert, update, delete on dbo.tcsa_AlertUsage to iMIS;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_AlertUsage')
		and name = 'icsa_AlertUsageUrlKey')
begin
	create index icsa_AlertUsageUrlKey
		on dbo.tcsa_AlertUsage(AlertUsageUrlKey);
end
go

if object_id('tcsa_AlertUsageUrl_CSAInstallTemp') is not NULL
begin
	set identity_insert tcsa_AlertUsageUrl on;

	insert dbo.tcsa_AlertUsageUrl(AlertUsageUrlKey, Url)
		select AlertUsageUrlKey, Url
		from tcsa_AlertUsageUrl_CSAInstallTemp
		order by AlertUsageUrlKey;

	if @@rowcount = (select count(*) from dbo.tcsa_AlertUsageUrl_CSAInstallTemp)
	begin
		drop table tcsa_AlertUsageUrl_CSAInstallTemp
		alter index all on tcsa_AlertUsageUrl rebuild;
	end
	else
	begin
		print 'Error copying and dropping tcsa_AlertUsageUrl_CSAInstallTemp'
	end

	set identity_insert tcsa_AlertUsageUrl off;

end
go
if object_id('tcsa_AlertUsage_CSAInstallTemp') is not NULL
	and exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertUsage_CSAInstallTemp' and name = 'Url')
begin
	insert dbo.tcsa_AlertUsageUrl(Url)
		select distinct Url
		from dbo.tcsa_AlertUsage_CSAInstallTemp

	alter index all on tcsa_AlertUsageUrl rebuild;

	exec ('
	insert dbo.tcsa_AlertUsage(EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, AlertUsageUrlKey)
		select aut.EventStart, aut.EventEnd, aut.RuleGroup, aut.RuleCount, aut.RowsCount, aut.ErrorCount,
			aut.ActionCount, aut.AutoActionCount, aut.ID, aut.UserId, auu.AlertUsageUrlKey
		from dbo.tcsa_AlertUsage_CSAInstallTemp aut
		join dbo.tcsa_AlertUsageUrl auu
			on aut.Url = auu.Url
		order by SEQN;')

	if @@rowcount = (select count(*) from dbo.tcsa_AlertUsage_CSAInstallTemp)
	begin
		drop table tcsa_AlertUsage_CSAInstallTemp
		alter index all on tcsa_AlertUsage rebuild;
	end
	else
	begin
		print 'Error migrating and dropping tcsa_AlertUsage_CSAInstallTemp'
	end
end
go
if object_id('tcsa_AlertUsage_CSAInstallTemp') is not NULL
	and exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_AlertUsage_CSAInstallTemp' and name = 'AlertUsageUrlKey')
begin
	exec ('
	insert dbo.tcsa_AlertUsage(EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, AlertUsageUrlKey)
		select EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, AlertUsageUrlKey
		from tcsa_AlertUsage_CSAInstallTemp
		order by SEQN;');

	if @@rowcount = (select count(*) from dbo.tcsa_AlertUsage_CSAInstallTemp)
	begin
		drop table tcsa_AlertUsage_CSAInstallTemp;
		alter index all on tcsa_AlertUsage rebuild;
	end
	else
	begin
		print 'Error copying and dropping tcsa_AlertUsage_CSAInstallTemp'
	end
end
go
if object_id('tcsa_AlertUsage_CSAInstallTemp') is not NULL
begin
	print 'Error during install:  tcsa_AlertUsage_CSAInstallTemp could not be migrated to current schema'
end
go

print 'Creating view vcsa_AlertUsage';
go
if object_id('vcsa_AlertUsage') is NULL
begin
	exec ('create /* model */ view vcsa_AlertUsage as select 1 as x;');
end
go
alter view vcsa_AlertUsage
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Combined view of tcsa_AlertUsage and related tables
select au.SEQN, au.EventStart, au.EventEnd, au.RuleGroup, au.RuleCount, au.RowsCount, au.ErrorCount,
		au.ActionCount, au.AutoActionCount, au.ID, au.UserId, auu.Url, auu.AlertUsageUrlKey
	from dbo.tcsa_AlertUsage au
	join dbo.tcsa_AlertUsageUrl auu
		on auu.AlertUsageUrlKey = au.AlertUsageUrlKey
go
grant select, insert, update, delete, references, view definition on vcsa_AlertUsage to IMIS;
go

print 'Creating procedure pcsa_WriteSystemParams (used by Xtender)';
go
if object_id('pcsa_WriteSystemParams') is NULL
begin
	exec ('create /* model */ procedure pcsa_WriteSystemParams as set nocount on;');
end;
go
alter procedure pcsa_WriteSystemParams
	@sParameterName varchar(50),
	@sShortValue varchar(255)
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
begin;
	set nocount on;

	if @sParameterName is not NULL
	begin;
		if @sShortValue is NULL
		begin;
			delete dbo.System_Params
				where ParameterName = @sParameterName;
		end;
		else
		begin;
			insert dbo.System_Params (ParameterName, ShortValue)
				select @sParameterName, @sShortValue
				where not exists (select 1 from dbo.System_Params where ParameterName = @sParameterName);

			if @@rowcount = 0
			begin;
				update sp
					set ShortValue = @sShortValue
					from dbo.System_Params sp
					where sp.ParameterName = @sParameterName;
			end;
		end;
	end;
end;
go
grant execute on dbo.pcsa_WriteSystemParams to IMIS;
go

print 'Dropping obsolete procedure pcsa_calculate_status';
go
if object_id('pcsa_calculate_status') is not NULL
begin;
	drop procedure dbo.pcsa_calculate_status;
end;
go

print 'Adding security keyword for CSA Editor window (used by Xtender)';
go
if not exists (select 1 from dbo.Gen_Tables where TABLE_NAME = 'ACCESS_KEYWORDS'
		and upper(CODE) = 'CSAEDIT')
begin;
	insert dbo.Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, DESCRIPTION)
		values ('ACCESS_KEYWORDS', 'CSAEdit', 'CSAEDIT', 'CSA Alert Editor');
end;
go

print 'Adding CSAEdit keyword to Manager, Administrator logins';
go
-- One row at a time to be as safe as possible with triggers
update dbo.Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'MANAGER'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255;
update dbo.Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'ADMINISTRATOR'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255;
go

print 'Creating table of TaskCentre servers, logins, passwords (used by Xtender)';
go
if not exists (select 1 from dbo.Gen_Tables where TABLE_NAME = 'CSA_TASKCENTRE_SERVERS')
begin;
	insert dbo.Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, SUBSTITUTE, DESCRIPTION)
		values ('CSA_TASKCENTRE_SERVERS', 'ServerAlias', 'SERVERALIAS', 'RealServerName;aTCLoginForCSA;TCPassword',
			'TaskCentre server for use by CSA. Code = nickname referenced by alert. Expansion ='
			+ ' server;user;password to be used by CSA separated by semicolons.');
end;
go

print 'Creating configuration items (iPart version)';
go

-- Insert RSM settings group if not present
insert dbo.SystemConfigPageRef (SystemConfigPageKey, SystemConfigPageName, SystemConfigPageDesc)
	select '316F8E3A-8391-45E4-9005-BC5FDE08EFAC', 'RSMCustomSettings', 'Custom settings for RSM-created features'
	where not exists (select 1 from dbo.SystemConfigPageRef where SystemConfigPageKey = '316F8E3A-8391-45E4-9005-BC5FDE08EFAC');
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF')
begin;
	print '-- adding RSM.CSA.License';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF', 'RSM.CSA.License',
		'85F7051F-0D92-4802-A6CC-74E9B54EA825', 7, 7, 40,
		'<span class="HelperText">Customer Service Alerts settings</span><br/><br/>License key for Customer Service Alerts iParts',
		'', 1101;
end;
else
begin;
	print '-- RSM.CSA.License already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1101
		where ParameterName = 'RSM.CSA.License';
	update dbo.SystemConfigParameterRef
		set Description = '<span class="HelperText">Customer Service Alerts settings</span><br/><br/>License key for Customer Service Alerts iParts'
		where ParameterName = 'RSM.CSA.License';
end;
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '2E23E97E-6D8A-4F3F-A59A-4C22E577849E')
begin;
	print '-- adding RSM.CSA.DefaultMarkupInOutput';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'2E23E97E-6D8A-4F3F-A59A-4C22E577849E', 'RSM.CSA.DefaultMarkupInOutput',
		'CC72633A-56F5-4D48-B6EF-2006EE876540', 2, 0, 40,
		'Allow unescaped HTML in alert output by default. May be overridden by iPart configuration or alert output.',
		'False', 1102;
end;
else
begin;
	print '-- RSM.CSA.DefaultMarkupInOutput already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1102
		where ParameterName = 'RSM.CSA.DefaultMarkupInOutput'
end;
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '72A671E1-CAFE-4234-8082-8C3CD203D38B')
begin;
	print '-- adding RSM.CSA.LogUsage';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'72A671E1-CAFE-4234-8082-8C3CD203D38B', 'RSM.CSA.LogUsage',
		'EBC22BE5-C57D-4202-904D-A1D280C6EA35', 2, 0, 40,
		'Write statistics to tcsa_AlertUsage each time a RuleGroup is evaluated.',
		'True', 1103;
end;
else
begin;
	print '-- RSM.CSA.LogUsage already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1103
		where ParameterName = 'RSM.CSA.LogUsage'

end;
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '1F68F287-264A-4D12-9248-5EF561811B5C')
begin;
	print '-- adding RSM.CSA.LogUsageGuest';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'1F68F287-264A-4D12-9248-5EF561811B5C', 'RSM.CSA.LogUsageGuest',
		'E3EF5715-BF74-4A22-99FB-EA88E3EB9435', 2, 0, 40,
		'Include usage statistics for anonymous users (GUEST)',
		'False', 1104;
end;
else
begin;
	print '-- RSM.CSA.LogUsageGuest already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1104
		where ParameterName = 'RSM.CSA.LogUsageGuest'
end;
go

if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '3b346e0d-6794-4993-9fce-aeb3769cc8af')
begin
	print '-- adding RSM.CSA.MaintenanceInterval';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'3b346e0d-6794-4993-9fce-aeb3769cc8af', 'RSM.CSA.MaintenanceInterval',
		'990b02a5-624b-4ef6-a79b-e591553af131', 6, 14, 20,
		'Minutes between executions of internal maintenance. Set shorter during development, longer in production.  0=off.',
		'240', 1105;
end
else
begin
	print '-- RSM.CSA.MaintenanceInterval already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1105
		where ParameterName = 'RSM.CSA.MaintenanceInterval';
end;
go
if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '009ae066-0757-4ff5-a786-63d80ff89a08')
begin
	print '-- adding RSM.CSA.Maint.PurgePageActions';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'009ae066-0757-4ff5-a786-63d80ff89a08', 'RSM.CSA.Maint.PurgePageActions',
		'6ab74302-d82a-42e2-a4d7-4b3c65ebd14e', 6, 14, 20,
		'Number of days to preserve actions generated by page. 0 = forever',
		'30', 1106;
end
else
begin
	print '-- RSM.CSA.Maint.PurgePageActions already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1106
		where ParameterName = 'RSM.CSA.Maint.PurgePageActions';
end;
go

if not exists (select 1 from dbo.SystemConfig where SystemConfigKey = '0b2cf809-f4a6-463f-9a3f-7549ff57e2d8')
begin
	print '-- adding RSM.CSA.Maint.PurgeActionCommands';

	exec dbo.asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'0b2cf809-f4a6-463f-9a3f-7549ff57e2d8', 'RSM.CSA.Maint.PurgeActionCommands',
		'f9de5e64-e890-4548-90d4-2249f2cce03b', 6, 14, 20,
		'Number of days to preserve CommandText from Actions since created or last executed. 0 = forever',
		'365', 1107;
end
else
begin
	print '-- RSM.CSA.Maint.PurgeActionCommands already exists';
	update dbo.SystemConfigPageParameterRef
		set SortOrder = 1107
		where ParameterName = 'RSM.CSA.Maint.PurgeActionCommands';
end;
go

print 'Creating table tcsa_EventsForTaskCentre (used by iPart)';
go
if object_id('tcsa_EventsForTaskCentre') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_EventsForTaskCentre') and indid < 2) > 0
	begin;
		print '***Not deleting - data exists';
	end;
	else
	begin;
		drop table dbo.tcsa_EventsForTaskCentre;
	end;
end;
go
if object_id('tcsa_EventsForTaskCentre') is NULL
begin;
	create table dbo.tcsa_EventsForTaskCentre
	(
		-- Automatically assigned properties
		Seqn int identity,
		InsertedDate datetime not NULL default getdate(),
		InsertedBy nvarchar(128) not NULL default user_name(),
		OriginDatabase nvarchar(128) not NULL default db_name(),

		-- Populated by CSA
		ServerAlias varchar(60) not NULL,			-- Server alias, for compatibility with CSA desktop
		TaskId int not NULL,						-- Task which will handle this event
		UserName varchar(60) not NULL,				-- User who clicked the Action button
		SelectedId varchar(50) not NULL,			-- Contact to which the Action applies
		Param1 nvarchar(255) not NULL default '',	-- Optional parameter 1
		Param2 nvarchar(255) not NULL default '',	-- Optional parameter 2
		Choice int not NULL default -1,				-- Optional Radio Button Choice selected

		-- Defaulted during insert, task may update
		ProcessedDate datetime NULL,
		ProcessedStatus int not NULL default 0,
		ProcessedMessage nvarchar(255) not NULL default ''
	);
end;
go
if not exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_EventsForTaskCentre' and name = 'Choice')
begin;
	alter table dbo.tcsa_EventsForTaskCentre
		add Choice int not NULL default -1;
end;
go
if object_id('pkcsa_EventsForTaskCentre') is NULL
begin;
	alter table dbo.tcsa_EventsForTaskCentre
		add constraint pkcsa_EventsForTaskCentre
		primary key clustered (Seqn);
end;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_EventsForTaskCentre')
		and name = 'icsa_EventsForTaskCentreSelectedId')
begin;
	create index icsa_EventsForTaskCentreSelectedId
		on dbo.tcsa_EventsForTaskCentre(SelectedId);
end;
go
grant select, insert, update, delete, references, view definition on dbo.tcsa_EventsForTaskCentre to IMIS;
grant select on dbo.tcsa_EventsForTaskCentre to public;
go

print 'Creating table tcsa_ActionPage';
go
if object_id('tcsa_ActionPage') is NULL
begin
	create table dbo.tcsa_ActionPage
	(
		ActionPageKey int not NULL identity(1,1),
		PageInstanceKey uniqueidentifier not NULL,
		CreatedDate datetime not NULL,
		LoggedInId varchar(10) not NULL,
		SubjectId varchar(10) not NULL,
	);
end;
go
grant select, insert, update, delete, references, view definition on tcsa_ActionPage to IMIS;
go
if object_id('pkcsa_ActionPage') is NULL
begin
	alter table tcsa_ActionPage
		add constraint pkcsa_ActionPage
		primary key clustered (ActionPageKey);
end;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_ActionPage')
		and name = 'icsa_ActionPage_PageInstanceKey')
begin
	create unique index icsa_ActionPage_PageInstanceKey
		on tcsa_ActionPage(PageInstanceKey);
end;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_ActionPage')
		and name = 'icsa_ActionPage_Created')
begin
	create index icsa_ActionPage_Created
		on tcsa_ActionPage(CreatedDate);
end;
go

print 'Creating table tcsa_ActionText';
go
if object_id('tcsa_ActionText') is NULL
begin
	create table dbo.tcsa_ActionText
	(
		ActionTextKey bigint not NULL identity(1,1),
		CreatedDate datetime not NULL,
		ExecuteCount int not NULL,
		LastExecuted datetime NULL,

		ActionType int not NULL,
		CommandText nvarchar(1000) not NULL,
		Digest as case
				when len(CommandText) <= 40 then left(CommandText, 40)
				else left(CommandText, 15) + right(CommandText, 25)
			end
	);
end;
go
grant select, insert, update, delete, references, view definition on tcsa_ActionText to IMIS;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_ActionText')
		and name = 'icsa_ActionTextDigest')
begin
	create clustered index icsa_ActionTextDigest
		on dbo.tcsa_ActionText(Digest);
end;
go
if object_id('pkr_ActionText') is NULL
begin
	alter table tcsa_ActionText
		add constraint pkr_ActionText
		primary key nonclustered (ActionTextKey);
end;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_ActionText')
		and name = 'icsa_ActionTextDates')
begin
	create index icsa_ActionTextDates
		on dbo.tcsa_ActionText(CreatedDate, LastExecuted)
		include (ActionTextKey);
end
go

print 'Creating table tcsa_ActionQueue (used by iPart)';
go
if object_id('tcsa_ActionQueue') is not NULL
begin;
	if (select rows from dbo.sysindexes where id = object_id('tcsa_ActionQueue') and indid < 2) = 0
	begin
		drop table tcsa_ActionQueue;
	end;
end;
go
if exists (select 1 from dbo.syscolumns where object_name(id) = 'tcsa_ActionQueue' and name = 'CommandText')
begin
	if object_id('dbo.pkcsa_ActionQueue') is not null
	begin
		alter table tcsa_ActionQueue
			drop constraint pkcsa_ActionQueue;
	end

	exec sp_rename 'tcsa_ActionQueue', 'tcsa_ActionQueue_CSAInstallTemp';
end
go
if object_id('tcsa_ActionQueue') is NULL
begin
	create table dbo.tcsa_ActionQueue
	(
		Seqn bigint identity
			constraint pkcsa_ActionQueue primary key clustered,
		ActionPageKey int not NULL
			constraint FK_ActionQueue_Page
				references dbo.tcsa_ActionPage(ActionPageKey)
				on delete cascade,
		ActionItemKey uniqueidentifier not NULL,
		ActionTextKey bigint not NULL
			constraint FK_ActionQueue_ActionTextKey
				references dbo.tcsa_ActionText(ActionTextKey)
				on delete cascade,
		RuleSeqn int not NULL
	);
end;
go
grant select, insert, update, delete on dbo.tcsa_ActionQueue to iMIS;
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_ActionQueue')
		and name = 'icsa_ActionQueueKeys')
begin
	create unique index icsa_ActionQueueKeys
		on dbo.tcsa_ActionQueue(ActionPageKey, ActionItemKey);
end
go
if not exists (select 1 from dbo.sysindexes where id = object_id('tcsa_ActionQueue')
		and name = 'icsa_ActionTextKey')
begin
	create index icsa_ActionTextKey
		on dbo.tcsa_ActionQueue(ActionTextKey);
end
go

-- If we have a table waiting to be converted
if object_id('tcsa_ActionQueue_CSAInstallTemp') is not null
begin;
	print 'Converting existing action queue to multiple tables'

	declare @rows int;

	select @rows = count(*)
		from dbo.tcsa_ActionQueue_CSAInstallTemp;

	insert dbo.tcsa_ActionText (CreatedDate, ExecuteCount, LastExecuted, ActionType, CommandText)
		select min(CreatedDate), sum(ExecuteCount), max(LastExecuted), ActionType, CommandText
		from dbo.tcsa_ActionQueue_CSAInstallTemp
		group by ActionType, CommandText
		order by min(CreatedDate);

	alter index all on tcsa_ActionText rebuild;

	insert dbo.tcsa_ActionPage (PageInstanceKey, CreatedDate, LoggedInId, SubjectId)
		select PageInstanceKey, max(CreatedDate), LoggedInId, SubjectId
		from tcsa_ActionQueue_CSAInstallTemp
		group by PageInstanceKey, LoggedInId, SubjectId;

	alter index all on tcsa_ActionPage rebuild;

	insert dbo.tcsa_ActionQueue (ActionPageKey, ActionItemKey, ActionTextKey, RuleSeqn)
		select ap.ActionPageKey, aqo.ActionItemKey, at.ActionTextKey, aqo.RuleSeqn
		from dbo.tcsa_ActionQueue_CSAInstallTemp aqo
		join dbo.tcsa_ActionText at
			on at.Digest = case
					when len(aqo.CommandText) <= 40 then left(aqo.CommandText, 40)
					else left(aqo.CommandText, 15) + right(aqo.CommandText, 25)
					end
			and at.ActionType = aqo.ActionType
			and at.CommandText = aqo.CommandText
		join dbo.tcsa_ActionPage ap
			on ap.PageInstanceKey = aqo.PageInstanceKey
		order by aqo.Seqn;

	if @@rowcount = @rows
	begin
		drop table tcsa_ActionQueue_CSAInstallTemp;
	end

	alter index all on tcsa_ActionQueue rebuild;
end;
go

print 'Creating view vcsa_ActionQueue';
go
if object_id('vcsa_ActionQueue') is NULL
begin
	exec ('create /* model */ view vcsa_ActionQueue as select 1 as x;');
end
go
alter view vcsa_ActionQueue
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Combine tcsa_ActionQueue and its child tables
select aq.Seqn, ap.PageInstanceKey, aq.ActionItemKey, ap.CreatedDate 'PageCreatedDate', ap.LoggedInId, ap.SubjectId,
		at.CreatedDate 'TextCreatedDate', at.ExecuteCount, at.LastExecuted, at.ActionType, at.CommandText, aq.RuleSeqn
	from dbo.tcsa_ActionQueue aq
	join dbo.tcsa_ActionPage ap
		on ap.ActionPageKey = aq.ActionPageKey
	join dbo.tcsa_ActionText at
		on at.ActionTextKey = aq.ActionTextKey;
go
grant select, insert, update, delete, references, view definition on vcsa_ActionQueue to IMIS;
go

print 'Creating procedure pcsa_Internal_SaveAction';
go
if object_id('pcsa_Internal_SaveAction') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_SaveAction as set nocount on;');
end;
go
alter procedure pcsa_Internal_SaveAction
	@PageInstanceKey uniqueidentifier,
	@ActionItemKey uniqueidentifier,
	@ActionType int,
	@LoggedInId varchar(10),
	@SubjectId varchar(10),
	@RuleSeqn int,
	@CommandText nvarchar(1000)
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Insert the provided action details to the action queue
begin;
	set nocount on;

	declare @Digest nvarchar(40),
		@rows int,
		@actionPageKey int,
		@actionTextKey bigint;

	-- Optimistic: assume it exists
	select @actionPageKey = ActionPageKey
		from dbo.tcsa_ActionPage ap
		where ap.PageInstanceKey = @PageInstanceKey;

	if @@rowcount = 0
	begin
		-- Insert, but avoid duplicate in case of timing issue
		insert dbo.tcsa_ActionPage (PageInstanceKey, CreatedDate, LoggedInId, SubjectId)
			select @PageInstanceKey, getdate(), @LoggedInId, @SubjectId
			where not exists (select 1 from dbo.tcsa_ActionPage where PageInstanceKey = @PageInstanceKey);

		select @rows = @@rowcount,
			@actionPageKey = @@identity;

		if @rows = 0
		begin
			-- If we weren't able to insert it, retrieve it again
			select @actionPageKey = ActionPageKey
				from dbo.tcsa_ActionPage ap
				where ap.PageInstanceKey = @PageInstanceKey;
		end
	end

	set @Digest = case
				when len(@CommandText) <= 40 then @CommandText
				else left(@CommandText, 15) + right(@CommandText, 25)
			end

	insert tcsa_ActionText(CreatedDate, ExecuteCount, LastExecuted, ActionType, CommandText)
		select getdate(), 0, null, @ActionType, @CommandText
		where not exists (select 1
				from dbo.tcsa_ActionText at
				where at.Digest = @Digest
					and at.ActionType = @ActionType
					and at.CommandText = @CommandText)

	select @actionTextKey = ActionTextKey
		from dbo.tcsa_ActionText at
		where at.Digest = @Digest
			and at.ActionType = @ActionType
			and at.CommandText = @CommandText

	insert dbo.tcsa_ActionQueue (ActionPageKey, ActionItemKey, ActionTextKey, RuleSeqn)
		values (@actionPageKey, @ActionItemKey, @ActionTextKey, @RuleSeqn);
end;
go
grant execute, view definition on dbo.pcsa_Internal_SaveAction to IMIS;
go

print 'Creating procedure pcsa_Internal_GroomActions';
go
if object_id('pcsa_Internal_GroomActions') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_GroomActions as set nocount on;');
end;
go
alter procedure pcsa_Internal_GroomActions
	@numDaysPages int = 30,
	@numDaysActions int = 365
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Placeholder for old procedure -- no longer used
begin
	set ansi_defaults on;
	set arithabort on;
	set concat_null_yields_null on;
	set implicit_transactions off;
	set numeric_roundabort off;
	set nocount on;

	print 'Obsolete:  CSA performs grooming automatically';
end;
go
grant execute, view definition on pcsa_Internal_GroomActions to IMIS;
go

print 'Creating procedure pcsa_Internal_Maintenance';
go
if object_id('pcsa_Internal_Maintenance') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_Maintenance as set nocount on;');
end;
go
alter procedure pcsa_Internal_Maintenance
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Delete page information and action text according to configured maintenance settings
begin;
	set nocount on;

	declare @numDaysPages int = 30,
		@xPagesCutoff datetime,
		@numDaysActions int = 365,
		@xActionTextCutoff datetime,
		@rows int


	set @rows = 0;

	select @numDaysPages = convert(int, ParameterValue)
		from dbo.SystemConfig
		where ParameterName = 'RSM.CSA.Maint.PurgePageActions'
			and isnumeric(ParameterValue) = 1

	if @numDaysPages != 0
	begin
		create table #deletedActionPages (ActionPageKey int)

		set @xPagesCutoff = dateadd(day, -1 * abs(@numDaysPages), getdate());

		delete ap
			output deleted.ActionPageKey into #deletedActionPages(ActionPageKey)
			from dbo.tcsa_ActionPage ap
			where ap.CreatedDate < @xPagesCutoff;

		set @rows = @rows + @@rowcount;

		delete aq
			from dbo.tcsa_ActionQueue aq
			join #deletedActionPages dap
				on dap.ActionPageKey = aq.ActionPageKey;

		set @rows = @rows + @@rowcount;
		
		drop table #deletedActionPages;
	end

	select @numDaysActions = convert(int, ParameterValue)
		from dbo.SystemConfig
		where ParameterName = 'RSM.CSA.Maint.PurgeActionCommands'
			and isnumeric(ParameterValue) = 1;

	if @numDaysActions != 0
	begin
		set @xActionTextCutoff = dateadd(day, -1 * abs(@numDaysActions), getdate());

		delete at
			from dbo.tcsa_ActionText at
			where at.CreatedDate < @xActionTextCutoff
				and at.LastExecuted is NULL;

		set @rows = @rows + @@rowcount;

		delete at
			from dbo.tcsa_ActionText at
			where at.CreatedDate < @xActionTextCutoff
				and at.LastExecuted is not NULL
				and at.LastExecuted < @xActionTextCutoff;

		set @rows = @rows + @@rowcount;
	end

	return @rows;
end;
go
grant execute, view definition on dbo.pcsa_Internal_Maintenance to IMIS;
go

print 'Creating procedure pcsa_Internal_ExecuteAction';
go
if object_id('pcsa_Internal_ExecuteAction') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_ExecuteAction as set nocount on;');
end;
go
alter procedure pcsa_Internal_ExecuteAction
	@PageInstanceKey uniqueidentifier,
	@ActionItemKey uniqueidentifier
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Update the queue, return the action details to the caller to be executed
begin;
	set nocount on;

	declare @actionType int,
		@message nvarchar(255),
		@loggedInId varchar(10),
		@subjectId varchar(10),
		@commandText nvarchar(1000),
		@seqn bigint;

	set @message = '';

	update at
		set ExecuteCount = ExecuteCount + 1,
			LastExecuted = getdate(),
			@loggedInId = ap.LoggedInId,
			@subjectId = ap.SubjectId,
			@seqn = aq.Seqn,
			@actionType = ActionType,
			@commandText = CommandText
		from dbo.tcsa_ActionText at
		join dbo.tcsa_ActionQueue aq
			on aq.ActionTextKey = at.ActionTextKey
		join dbo.tcsa_ActionPage ap
			on ap.ActionPageKey = aq.ActionPageKey
		where ap.PageInstanceKey = @pageInstanceKey
			and aq.ActionItemKey = @actionItemKey

	if @@rowcount != 1
	begin;
		set @actionType = 0;
		set @message = 'No matching action found';
		goto done;
	end;
done:
	select @message 'ReturnMessage', @actionType 'ActionType', @loggedInId 'LoggedInId', @subjectId 'SubjectId',
		@commandText 'CommandText', @seqn 'Seqn';
end;
go
grant execute, view definition on dbo.pcsa_Internal_ExecuteAction to IMIS;
go

if object_id('pr_Internal_SaveUsage') is not NULL
begin
	exec sp_rename 'pr_Internal_SaveUsage', 'pcsa_Internal_SaveUsage';
end;
go
print 'Creating procedure pcsa_Internal_SaveUsage';
go
if object_id('pcsa_Internal_SaveUsage') is NULL
begin
	exec ('create /* model */ procedure pcsa_Internal_SaveUsage as set nocount on;');
end;
go
alter procedure pcsa_Internal_SaveUsage
	@eventStart datetime,
	@eventEnd datetime,
	@ruleGroup varchar(100),
	@ruleCount int,
	@rowsCount int,
	@errorCount int,
	@actionCount int,
	@autoActionCount int,
	@id varchar(10),
	@userId varchar(60),
	@url nvarchar(1000)
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Log an event each time a rulegroup is executed
begin
	set nocount on;

	declare @Digest nvarchar(40),
		@urlKey int;

	set @Digest = case
			when len(@url) <= 40 then @url
			else left(@url, 15) + right(@url, 25)
		end;

	insert tcsa_AlertUsageUrl (Url)
		select @url
		where not exists (select 1
				from dbo.tcsa_AlertUsageUrl auu
				where auu.Digest = @Digest
					and auu.Url = @url);

	select @urlKey = AlertUsageUrlKey
		from dbo.tcsa_AlertUsageUrl auu
		where auu.Digest = @Digest
			and auu.Url = @url;

	insert tcsa_AlertUsage (EventStart, EventEnd, RuleGroup, RuleCount, RowsCount, ErrorCount,
			ActionCount, AutoActionCount, ID, UserId, AlertUsageUrlKey)
		values (@eventStart, @eventEnd, @ruleGroup, @ruleCount, @rowsCount, @errorCount,
			@actionCount, @autoActionCount, @id, @userId, @urlKey)
end;
go
grant execute, view definition on dbo.pcsa_Internal_SaveUsage to IMIS;
go

print 'Creating procedure pr_csa_ExportAlertRules';
go
if object_id('pr_csa_ExportAlertRules') is NULL
begin
	exec ('create /* model */ procedure pr_csa_ExportAlertRules as set nocount on;');
end;
go
alter procedure pr_csa_ExportAlertRules
as
/***************************************************************************************************
	© 2022 RSM US LLP  All rights reserved.

RSM US LLP (RSM) grants to Client a limited, non-exclusive, non-transferable right to install and
use the Software/Source Code solely for internal business purposes.  Client may not distribute,
give, rent, loan, or resell a copy of the Software/Source Code to any third party. RSM is the owner
of the copyright for the Software/Source Code and RSM reserves all rights related to such ownership.
Any Warranty granted by RSM is void if Software/Source Code is modified by anyone other than RSM.

$URL: https://svn.rsm.cloud/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 21130 $
$Date: 2022-10-17 12:56:23 -0500 (Mon, 17 Oct 2022) $
***************************************************************************************************/
-- Export CSA alert definitions as a SQL script which can be run on another database
begin
	set nocount on;

	declare @criteria varchar(3072),
		@enabled bit,
		@description varchar(255),
		@alerttype varchar(10),
		@alertid varchar(20),
		@rulegroup varchar(100),

		@criteriasafe varchar(6144),
		@descriptionsafe varchar(510),
		@alerttypesafe varchar(20),
		@alertidsafe varchar(40),
		@rulegroupsafe varchar(200)

	create table #script (SEQN int identity, SCRIPT varchar(8000) not NULL)

	declare rule_cursor cursor local fast_forward
	for
	select CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup
		from dbo.tcsa_AlertDefinitions
		where 1 = 1
		order by RuleGroup, CRITERIA, SEQN;

	open rule_cursor;
	fetch next from rule_cursor
		into @criteria, @enabled, @description, @alerttype, @alertid, @rulegroup;

	while @@fetch_status <> -1
	begin
		if @@fetch_status = 0
		begin
			set @criteriasafe = replace(@criteria, '''', '''''');
			set @descriptionsafe = replace(@description, '''', '''''');
			set @alerttypesafe = replace(@alerttype, '''', '''''');
			set @alertidsafe = replace(@alertid, '''', '''''');
			set @rulegroupsafe = replace(@rulegroup, '''', '''''');

			insert #script (SCRIPT) values ('if not exists (select 1 from dbo.tcsa_AlertDefinitions where RuleGroup = ''' + @rulegroupsafe + '''')
			insert #script (SCRIPT) values ('	and CRITERIA = ''' + @criteriasafe + ''')')
			insert #script (SCRIPT) values ('begin')
			insert #script (SCRIPT) values ('	insert dbo.tcsa_AlertDefinitions (RuleGroup, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, CRITERIA)')
			insert #script (SCRIPT) values ('		values (''' + @rulegroupsafe + ''', ' + case @enabled when 1 then '1' else '0' end + ', '''
					+ @descriptionsafe + ''', ''' + @alerttypesafe + ''', ''' + @alertidsafe + ''',')
			insert #script (SCRIPT) values ('			''' + @criteriasafe + ''');')
			insert #script (SCRIPT) values ('end;')
			insert #script (SCRIPT) values ('')
		end;
		fetch next from rule_cursor
			into @criteria, @enabled, @description, @alerttype, @alertid, @rulegroup;
	end;
	close rule_cursor;
	deallocate rule_cursor;

	select SCRIPT
		from #script
		order by SEQN
end;
go
grant execute, view definition on dbo.pr_csa_ExportAlertRules to IMIS;
go

print 'Finished.';
go
